@@ -1,4 +1,4 @@
-/* $Id: details.cc,v 1.8 2000/04/15 14:12:32 bergo Exp $ */
+/* $Id: details.cc,v 1.9 2001/04/17 04:11:54 bergo Exp $ */
 
 /*
 
@@ -70,16 +70,20 @@ void open_details(gint thepid,char *thehost) {
 
   hs=gtk_hseparator_new();
   gtk_box_pack_start(GTK_BOX(v1),hs,FALSE,FALSE,4);
-  h1=gtk_hbox_new(FALSE,2);
-  gtk_box_pack_start(GTK_BOX(v1),h1,FALSE,TRUE,2);
 
-  dis=gtk_button_new_with_label("   Dismiss   ");
-  gtk_box_pack_end(GTK_BOX(h1),dis,FALSE,FALSE,2);
+  h1=gtk_hbutton_box_new();
+  gtk_button_box_set_layout(GTK_BUTTON_BOX(h1), GTK_BUTTONBOX_END);
+  gtk_button_box_set_spacing(GTK_BUTTON_BOX(h1), 5);
+
+  gtk_box_pack_start(GTK_BOX(v1),h1,FALSE,FALSE,6);
+
+  dis=gtk_button_new_with_label("Dismiss");
+  GTK_WIDGET_SET_FLAGS(dis,GTK_CAN_DEFAULT);
+  gtk_box_pack_start(GTK_BOX(h1),dis,TRUE,TRUE,0);
 
   gtk_signal_connect(GTK_OBJECT(dis),"clicked",
 		     GTK_SIGNAL_FUNC(dismiss_details),NULL);
 
-  GTK_WIDGET_SET_FLAGS(dis,GTK_CAN_DEFAULT);
   gtk_widget_grab_default(dis);
 
   gtk_widget_show(v1);
