@@ -1,9 +1,9 @@
-/* $Id: gps.cc,v 1.23 2001/02/23 13:48:05 bergo Exp $ */
+/* $Id: gps.cc,v 1.30 2002/04/28 19:33:21 bergo Exp $ */
 
 /*
 
     GPS - Graphical Process Statistics
-    Copyright (C) 1999-2000 Felipe Paulo Guazzi Bergo
+    Copyright (C) 1999-2002 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -50,11 +50,13 @@
 #include "nativepollers.h"
 #include "netwatch.h"
 #include "netpolling.h"
+#include "quickbar.h"
 #include "renice.h"
 #include "tabledata.h"
 #include "treeview.h"
 #include "tstring.h"
 #include "userbox.h"
+#include "watchp.h"
 
 #include "pixmaps/xray-tux.xpm"
 #include "pixmaps/wheel.xpm"
@@ -112,7 +114,9 @@ static char smonth[12][4]={"Jan","Feb","Mar","Apr","May","Jun",
 			   "Jul","Aug","Sep","Oct","Nov","Dec"};
 
 int delete_only_once=0,got_a_poller=0,wanttop=0;
-			   
+			
+int quickbar=0;
+   
 int main (int argc, char *argv[])
 {
   int i,j;
@@ -150,8 +154,43 @@ int main (int argc, char *argv[])
   }
 
   init_default_colors();
+  read_rc();
 
   for(i=1;i<argc;i++) {
+    if (strcmp(argv[i],"-qb")==0) {
+      quickbar=TRUE;
+
+      while ((argc-i)>1) {
+	if (!strcmp(argv[i+1],"top")) {
+	  quickbar_pos=QBAR_TOP;    ++i; continue;
+	}
+	if (!strcmp(argv[i+1],"bottom")) {
+	  quickbar_pos=QBAR_BOTTOM; ++i; continue;
+	}
+	if (!strcmp(argv[i+1],"left")) {
+	  quickbar_halign=QBAR_LEFT; ++i; continue;
+	}
+	if (!strcmp(argv[i+1],"right")) {
+	  quickbar_halign=QBAR_RIGHT; ++i; continue;
+	}
+	if (!strcmp(argv[i+1],"center")) {
+	  quickbar_halign=QBAR_CENTER; ++i; continue;
+	}
+	if (!strcmp(argv[i+1],"-w")) {
+	  ++i;
+	  if ((argc-i)>1) quickbar_width=atoi(argv[++i]);
+	  continue;
+	}
+	if (!strcmp(argv[i+1],"-r")) {
+	  quickbar_windowed=TRUE;
+	  ++i;
+	  continue;
+	}
+	break;
+      } // while argc-i > 1
+
+      continue;
+    } // if argv[] == -qb
     if (strcmp(argv[i],"-gb")==0) {
       jelly_bar=TRUE;
       continue;
@@ -196,7 +235,7 @@ int main (int argc, char *argv[])
     }
 
     if ((strcmp(argv[i],"--version")==0)||(strcmp(argv[i],"-v")==0)) {
-      printf("gPS version %s  (C) 1999-2000 Felipe Bergo <bergo@seul.org>\n",
+      printf("gPS version %s  (C) 1999-2001 Felipe Bergo <bergo@seul.org>\n",
 	     GPS_RELEASE);
       printf("Distributed under the terms of the"\
 	     " GNU General Public License\n\n");
@@ -209,6 +248,7 @@ int main (int argc, char *argv[])
   }
     
   gtk_init (&argc, &argv);
+  gdk_rgb_init(); /* for history gauges */
 
   if (dashc)
     continuum=TRUE;
@@ -216,24 +256,27 @@ int main (int argc, char *argv[])
   if (dashl)
     show_long_names=TRUE;
 
-  read_rc();
-
 #ifdef ENABLE_NETWORK
-  if (!nostartingnet)
+  if ((!nostartingnet)&&(!quickbar))
     netwatch_init();
 #endif
 
   init_local_polling();
-  init_main_window ();
-  nice(8);
-  gotgui=1;
-  refresh();
-  gdk_rgb_init(); /* for history gauges */
+
+
+  if (!quickbar) {
+    init_main_window ();
+    nice(8);
+    refresh();
+  } else
+    init_quickbar();
+
   gtk_main ();
 
   gotgui=0;
+  if (!quickbar)
+    netwatch_destroy();
 
-  netwatch_destroy();
   close_local_polling();
 
   return 0;
@@ -299,10 +342,13 @@ void init_main_window() {
   int i;
   char *buffer,*bu2;
 
+  gotgui=1;
+
   buffer=(char *)g_malloc(512);
   bu2=(char *)g_malloc(512);
   gethostname(bu2,512);
-  sprintf(buffer,"gPS: Process List (%s)",bu2);
+  snprintf(buffer,512,"gPS: Process List (%s)",bu2);
+  buffer[511]=0;
 
   /* MAIN WINDOW */
   mainw=gtk_window_new(GTK_WINDOW_TOPLEVEL);
@@ -385,7 +431,7 @@ void init_main_window() {
   gtk_signal_connect (GTK_OBJECT (mi[8]), "activate",
 		      GTK_SIGNAL_FUNC (about_gps), NULL);
   gtk_signal_connect (GTK_OBJECT (mi[9]), "activate",
-		      GTK_SIGNAL_FUNC (destroy), NULL);
+		      GTK_SIGNAL_FUNC (pdestroy), NULL);
 
   /* Filter */
   n=gtk_menu_new();
@@ -413,6 +459,10 @@ void init_main_window() {
   mi[11]=gtk_menu_item_new_with_label("Renice...");
   mi[12]=gtk_menu_item_new();
   mi[13]=gtk_menu_item_new_with_label("Details...");
+  mi[14]=gtk_menu_item_new();
+  mi[15]=gtk_menu_item_new_with_label("Watch Process: half second polling");
+  mi[16]=gtk_menu_item_new_with_label("Watch Process: 5 second polling");
+  mi[17]=gtk_menu_item_new_with_label("Watch Process: 1 minute polling");
 
   sc[0]=mi[7];
   sc[1]=mi[8];
@@ -424,7 +474,8 @@ void init_main_window() {
   buffer=(char *)g_malloc(128);
   
   for(i=0;i<POSIXSIG;i++) {
-    sprintf(buffer,"%s (%d)",posixnames[i],posixvalues[i]);
+    snprintf(buffer,128,"%s (%d)",posixnames[i],posixvalues[i]);
+    buffer[127]=0;
     mj[i+1]=gtk_menu_item_new_with_label(buffer);
     gtk_menu_append(GTK_MENU(mj[0]),mj[i+1]);
 
@@ -453,9 +504,9 @@ void init_main_window() {
   gtk_accel_group_add(mmag,GDK_N,GDK_CONTROL_MASK,GTK_ACCEL_VISIBLE,
 		      GTK_OBJECT(mi[11]),"activate");
 
-  for(i=7;i<14;i++)
+  for(i=7;i<18;i++)
     gtk_menu_append(GTK_MENU(o),mi[i]);
-  for(i=7;i<14;i++) {
+  for(i=7;i<18;i++) {
     gtk_widget_lock_accelerators(mi[i]);
     gtk_widget_show(mi[i]);
   }
@@ -471,6 +522,12 @@ void init_main_window() {
 		      GTK_SIGNAL_FUNC (renice_dialog), NULL);
   gtk_signal_connect (GTK_OBJECT (mi[13]), "activate",
 		      GTK_SIGNAL_FUNC (show_details), NULL);
+  gtk_signal_connect (GTK_OBJECT (mi[15]), "activate",
+		      GTK_SIGNAL_FUNC (watch_process), (gpointer) "500");
+  gtk_signal_connect (GTK_OBJECT (mi[16]), "activate",
+		      GTK_SIGNAL_FUNC (watch_process), (gpointer) "5000");
+  gtk_signal_connect (GTK_OBJECT (mi[17]), "activate",
+		      GTK_SIGNAL_FUNC (watch_process), (gpointer) "60000");
 
   /* Network */
   net=gtk_menu_new();
@@ -563,7 +620,7 @@ void init_main_window() {
   gtk_clist_set_selection_mode(GTK_CLIST(pl),GTK_SELECTION_SINGLE);
   for(i=0;i<PROCESS_LIST_COLUMNS;i++)
     gtk_clist_set_column_title(GTK_CLIST(pl),i,(gchar *)(col_title[i]));
-  gtk_clist_column_titles_passive(GTK_CLIST(pl));
+  gtk_clist_column_titles_active(GTK_CLIST(pl));
   gtk_clist_column_titles_show(GTK_CLIST(pl));
 
   for(i=5;i<=9;i++)
@@ -689,7 +746,7 @@ void init_main_window() {
   gtk_signal_connect (GTK_OBJECT (b4), "clicked",
 		      GTK_SIGNAL_FUNC (kill_with_sigkill), NULL);
   gtk_signal_connect (GTK_OBJECT (b3), "clicked",
-		      GTK_SIGNAL_FUNC (destroy), NULL);
+		      GTK_SIGNAL_FUNC (pdestroy), NULL);
   gtk_signal_connect (GTK_OBJECT(pl),"select_row",
 		      GTK_SIGNAL_FUNC(selection_callback),NULL);
   gtk_signal_connect (GTK_OBJECT(pl),"unselect_row",
@@ -716,13 +773,17 @@ gint kill_main (GtkWidget * widget, GdkEvent * event, gpointer data)
   return FALSE;
 }
 
+void pdestroy (GtkWidget * widget, gpointer data) {
+  gtk_widget_destroy(GTK_WIDGET(MainWindow));
+}
+
 void destroy (GtkWidget * widget, gpointer data)
 {
   if (to_tag!=-1) { /* kill refresh timer */
     gtk_timeout_remove(to_tag);
     to_tag=-1;
   }
-  while(!enter_mutex()) usleep(140); // hope this doesn't lock
+  while(!enter_mutex()) usleep(140);
   gtk_main_quit();
 }
 
@@ -816,7 +877,8 @@ void refresh() {
   update_list_widget();
   gtk_clist_thaw(GTK_CLIST(psl));
 
-  sprintf(buffer,"%d processes, %d shown",nproc,nproc-sproc);
+  snprintf(buffer,256,"%d processes, %d shown",nproc,nproc-sproc);
+  buffer[255]=0;
   gtk_label_set_text(GTK_LABEL(status_label[1]),buffer);
   g_list_free(biglist);
 }
@@ -864,6 +926,8 @@ void refresh_cb (GtkWidget * widget, gpointer data) {
 
 gint refresh_to (gpointer data) {
 
+  if (!gotgui)
+    return FALSE;
   if (!enter_mutex())
     return FALSE;
   refresh();
@@ -939,7 +1003,7 @@ void about_gps(GtkWidget *,gpointer) {
   char about[1024];
   int i;
 
-  GtkWidget *v,*h,*h2,*p,*text,*b;
+  GtkWidget *v,*h,*h2,*text,*b,*hs;
 
   dlg=gtk_window_new(GTK_WINDOW_DIALOG);
   gtk_window_set_transient_for(GTK_WINDOW(dlg),MainWindow);
@@ -969,7 +1033,7 @@ void about_gps(GtkWidget *,gpointer) {
   strcat(about,"Polling methods compiled:\n");
 
 #ifdef HAVELINUXPROCFS
-  strcat(about,"Linux 2.[0-4] native /proc fs poller\n");
+  strcat(about,"Linux 2.[0-5] native /proc fs poller\n");
 #endif
 
 #ifdef HAVEFREEBSD
@@ -994,21 +1058,25 @@ void about_gps(GtkWidget *,gpointer) {
   gtk_box_pack_start(GTK_BOX(h),text,FALSE,TRUE,2);
   gtk_label_set_justify(GTK_LABEL(text),GTK_JUSTIFY_LEFT);
 
-  h2=gtk_hbox_new(TRUE,2);
-  gtk_box_pack_start(GTK_BOX(v),h2,FALSE,TRUE,2);
+  hs=gtk_hseparator_new();
+  gtk_box_pack_start(GTK_BOX(v),hs,FALSE,FALSE,6);
 
-  for(i=0;i<3;i++) {
-    p=gtk_label_new(" ");
-    gtk_box_pack_start(GTK_BOX(h2),p,FALSE,FALSE,2);
-    gtk_widget_show(p);
-  }
+  h2=gtk_hbutton_box_new();
+  gtk_button_box_set_layout(GTK_BUTTON_BOX(h2), GTK_BUTTONBOX_END);
+  gtk_button_box_set_spacing(GTK_BUTTON_BOX(h2), 5);
+
+  gtk_box_pack_start(GTK_BOX(v),h2,FALSE,FALSE,6);
 
   b=gtk_button_new_with_label("Dismiss");
-  gtk_box_pack_start(GTK_BOX(h2),b,FALSE,TRUE,4);
+  GTK_WIDGET_SET_FLAGS(b,GTK_CAN_DEFAULT);
+  gtk_box_pack_start(GTK_BOX(h2),b,TRUE,TRUE,0);
 
   gtk_container_set_border_width(GTK_CONTAINER(dlg),6);
 
+  gtk_widget_grab_default(b);
+
   gtk_widget_show(b);
+  gtk_widget_show(hs);
   gtk_widget_show(text);
   gtk_widget_show(xrayp);
   gtk_widget_show(h2);
@@ -1255,6 +1323,12 @@ char * power_of_two_suffix(gint expon) {
   return(&sarray[i]);
 }
 
+void watch_process(GtkWidget *a, gpointer b) {
+  int interval;
+  interval=atoi( (const char *) b);
+  open_watch(pid_sel, host_sel, interval);
+}
+
 void show_details(GtkWidget *a,gpointer b) {
   open_details(pid_sel,host_sel);
 }
@@ -1323,7 +1397,8 @@ void pre_sort() {
 
 char *get_rc_path() {
   static char buffer[512];
-  sprintf(buffer,"%s/%s",getenv("HOME"),GPS_RC);
+  snprintf(buffer,512,"%s/%s",getenv("HOME"),GPS_RC);
+  buffer[511]=0;
   return(buffer);
 }
 
@@ -1502,6 +1577,7 @@ gboolean wheel_configure_event(GtkWidget *widget,GdkEventConfigure *ce,
 		  wheel_state%22,0,1,2+(y-20)/2,62,16);
 
   gdk_pixmap_unref(whole_wheel);
+  return FALSE;
 }
 
 void walk_wheel() {
@@ -1562,7 +1638,7 @@ GtkWidget *s_dlg,*vf_cb[PROCESS_LIST_COLUMNS],*intv1,*intv2;
 
 void settings(GtkWidget *w,gpointer data) {
   GtkWidget *fr1,*fr2,*tv,*tb,*e1,*e2,*l1,*l2,
-            *v1,*hs,*okb,*cab,*h1,*f1,
+            *v1,*hs,*okb,*cab,*h1,
             *hb[PROCESS_LIST_COLUMNS];
   char z[64];
   int i;
@@ -1625,21 +1701,20 @@ void settings(GtkWidget *w,gpointer data) {
   hs=gtk_hseparator_new();
   gtk_box_pack_start(GTK_BOX(tv),hs,FALSE,TRUE,3);
 
-  h1=gtk_hbox_new(TRUE,8);
+  h1=gtk_hbutton_box_new();
+  gtk_button_box_set_layout(GTK_BUTTON_BOX(h1), GTK_BUTTONBOX_END);
+  gtk_button_box_set_spacing(GTK_BUTTON_BOX(h1), 5);
 
-  f1=gtk_label_new(" ");
-  gtk_box_pack_start(GTK_BOX(h1),f1,FALSE,TRUE,0);
-  gtk_widget_show(f1);
-  f1=gtk_label_new(" ");
-  gtk_box_pack_start(GTK_BOX(h1),f1,FALSE,TRUE,0);
-  gtk_widget_show(f1);
+  okb=gtk_button_new_with_label("Ok");
+  GTK_WIDGET_SET_FLAGS(okb,GTK_CAN_DEFAULT);
+  cab=gtk_button_new_with_label("Cancel");
+  GTK_WIDGET_SET_FLAGS(cab,GTK_CAN_DEFAULT);
 
-  okb=gtk_button_new_with_label(" Ok ");
-  cab=gtk_button_new_with_label(" Cancel ");
+  gtk_box_pack_start(GTK_BOX(tv),h1,FALSE,FALSE,6);
+  gtk_box_pack_start(GTK_BOX(h1),okb,TRUE,TRUE,0);
+  gtk_box_pack_start(GTK_BOX(h1),cab,TRUE,TRUE,0);
 
-  gtk_box_pack_start(GTK_BOX(h1),okb,FALSE,TRUE,0);
-  gtk_box_pack_start(GTK_BOX(h1),cab,FALSE,TRUE,0);
-  gtk_box_pack_start(GTK_BOX(tv),h1,FALSE,TRUE,2);
+  gtk_widget_grab_default(okb);
 
   gtk_widget_show(hs);
   gtk_widget_show(cab);
@@ -1667,9 +1742,11 @@ void settings(GtkWidget *w,gpointer data) {
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(vf_cb[i]),
 				 ((visible_fields[i])?TRUE:FALSE));
 
-  sprintf(z,"%d",list_interval);
+  snprintf(z,64,"%d",list_interval);
+  z[63]=0;
   gtk_entry_set_text(GTK_ENTRY(e1),z);
-  sprintf(z,"%d",history_interval);
+  snprintf(z,64,"%d",history_interval);
+  z[63]=0;
   gtk_entry_set_text(GTK_ENTRY(e2),z);
   intv1=e1;
   intv2=e2;
