@@ -1,9 +1,9 @@
-/* $Id: linuxprocfs.cc,v 1.12 2000/10/03 03:35:21 bergo Exp $ */
+/* $Id: linuxprocfs.cc,v 1.15 2002/03/10 18:37:10 bergo Exp $ */
 
 /*
 
     GPS - Graphical Process Statistics
-    Copyright (C) 1999-2000 Felipe Paulo Guazzi Bergo
+    Copyright (C) 1999-2002 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -105,7 +105,7 @@ void LinuxProcFsProcessListPoller::poll() {
     t->set(buffer);
     if (t->contains("btime ")!=NULL) {
       t->token(T_SNT);
-      btime=atol(t->token(T_SNT));
+      btime=gps_atoul(t->token(T_SNT));
     }
     if (t->contains("processes ")!=NULL) {
       t->token(T_SNT);
@@ -131,7 +131,8 @@ void LinuxProcFsProcessListPoller::poll() {
       continue;
     }
     
-    sprintf(buffer,"/proc/%d/stat",i);
+    snprintf(buffer,1024,"/proc/%d/stat",i);
+    buffer[1023]=0;
 
     f=fopen(buffer,"r");
     if (f==NULL) {
@@ -145,7 +146,8 @@ void LinuxProcFsProcessListPoller::poll() {
     pli=new ProcessListItem();
 
     /* get owner */
-    sprintf(buffer,"/proc/%d/status",i);
+    snprintf(buffer,1024,"/proc/%d/status",i);
+    buffer[1023]=0;
     g=fopen(buffer,"r");
     pli->setField(OWNER,"???");
     if (g!=NULL) {
@@ -161,14 +163,22 @@ void LinuxProcFsProcessListPoller::poll() {
 	j=atoi(t->token(" \t"));
 	fclose(g);
 	ident=getpwuid(j);
-	pli->setField(OWNER,ident->pw_name);
+	if (ident) {
+	  pli->setField(OWNER,ident->pw_name);
+	} else {
+	  char tmpname[64];
+	  snprintf(tmpname,64,"#%d",j);
+	  tmpname[63]=0;
+	  pli->setField(OWNER,tmpname);
+	}
 	break;
       } /* while 1 */
     } /* g!=NULL */
 
 
     /* PID */
-    sprintf(b3,"%d",i);
+    snprintf(b3,512,"%d",i);
+    b3[511]=0;
     pli->setField(PID,b3);
     strcpy(buffer,wantmore);
     strcpy(buffercopy,buffer);
@@ -196,23 +206,25 @@ void LinuxProcFsProcessListPoller::poll() {
       t->token(" \t\n");
 
     /* CPU */
-    ulv=(unsigned long)atol(t->token(T_SNT));
-    ulv+=(unsigned long)atol(t->token(T_SNT));
-    dlv=(unsigned long)atol(t->token(T_SNT));
-    dlv+=(unsigned long)atol(t->token(T_SNT));
+    ulv= gps_atoul(t->token(T_SNT));
+    ulv+=gps_atoul(t->token(T_SNT));
+    dlv= gps_atoul(t->token(T_SNT));
+    dlv+=gps_atoul(t->token(T_SNT));
 
     /* PRIORITY & NICE */
     rib=atoi(t->token(T_SNT));
-    sprintf(b3,"%d",rib);
+    snprintf(b3,512,"%d",rib);
+    b3[511]=0;
     pli->setField(PRIORITY,b3);
     rib=atoi(t->token(T_SNT));
-    sprintf(b3,"%d",rib);
+    snprintf(b3,512,"%d",rib);
+    b3[511]=0;
     pli->setField(NICE,b3);
 
     for(j=0;j<2;j++)
       t->token(T_SNT);
 
-    rtime=(unsigned long)atol(t->token(T_SNT));
+    rtime=gps_atoul(t->token(T_SNT));
     ptime=btime+(time_t)(rtime/100UL);
 
     strcpy(b3,ctime(&ptime));
@@ -225,22 +237,25 @@ void LinuxProcFsProcessListPoller::poll() {
     if (dlv==0)
       dlv=1;
     cpufrac=((double)ulv)/((double)dlv);
-    sprintf(b3,"%.1f",cpufrac);
+    snprintf(b3,512,"%.1f",cpufrac);
+    b3[511]=0;
     pli->setField(CPU,b3);
 
     /* MEM (VSIZE) */
-    smem[0]=(unsigned long)atol(t->token(T_SNT));
+    smem[0]=gps_atoul(t->token(T_SNT));
     mem_unit=0;
     while(smem[0]>(99UL<<10)) {
       ++mem_unit;
       smem[0]>>=10;
     }
-    sprintf(b3,"%lu",smem[0]);
+    /* 500 used on purpose instead of 512, to leave room for the suffix */
+    snprintf(b3,500,"%lu",smem[0]);
+    b3[499]=0;
     strcat(b3,power_of_two_suffix(mem_unit));
     pli->setField(SIZE,b3);
 
     /* RSS */
-    smem[1]=(unsigned long)atol(t->token(T_SNT));
+    smem[1]=gps_atoul(t->token(T_SNT));
 
     /* STATE (2/2) */
     if (smem[1]==0)
@@ -260,12 +275,15 @@ void LinuxProcFsProcessListPoller::poll() {
       ++mem_unit;
       smem[1]>>=10;
     }
-    sprintf(b3,"%lu",smem[1]);
+    /* 500 used on purpose to leave room for the suffix */
+    snprintf(b3,500,"%lu",smem[1]);
+    b3[499]=0;
     strcat(b3,power_of_two_suffix(mem_unit));
     pli->setField(RSS,b3);
 
     /* LONG NAMES */
-    sprintf(buffer,"/proc/%d/cmdline",i);
+    snprintf(buffer,1024,"/proc/%d/cmdline",i);
+    buffer[1023]=0;
     f=fopen(buffer,"r");
     if (f!=NULL) {
       memset(buffer,0,1024);
@@ -312,13 +330,15 @@ void LinuxProcFsProcessDetailsPoller::poll(int whichpid) {
   reset();
   item=new ProcessItem();
 
-  sprintf(b2,"%d",whichpid);
+  snprintf(b2,128,"%d",whichpid);
+  b2[127]=0;
   item->setField(PID,b2);
 
   /* HOST */
   item->setField(MACHINE,get_local_hostname());
 
-  sprintf(buffer,"/proc/%d/stat",whichpid);
+  snprintf(buffer,1024,"/proc/%d/stat",whichpid);
+  buffer[1023]=0;
   f=fopen(buffer,"r");
   if (f==NULL) {
     reset();
@@ -338,14 +358,16 @@ void LinuxProcFsProcessDetailsPoller::poll(int whichpid) {
     t->set(buffer);
     if (t->contains("btime ")!=NULL) {
       t->token(T_SNT);
-      btime=atol(t->token(T_SNT));
+      btime=gps_atoul(t->token(T_SNT));
       break;
     }
   } 
   fclose(f);
 
   // user
-  sprintf(b2,"/proc/%d/status",whichpid);
+  snprintf(b2,128,"/proc/%d/status",whichpid);
+  b2[127]=0;
+
   strcpy(b3,"???");
   f=fopen(b2,"r");
   if (f!=NULL) {
@@ -360,11 +382,18 @@ void LinuxProcFsProcessDetailsPoller::poll(int whichpid) {
     t->token(T_SNT);
     i=atoi(t->token(T_SNT));
     ident=getpwuid(i);
-    item->setField(OWNER,ident->pw_name);
+    if (ident) {
+      item->setField(OWNER,ident->pw_name);
+    } else {
+      char tmpname[64];
+      snprintf(tmpname,64,"#%d",i);
+      item->setField(OWNER,tmpname);
+    }
   }
 
   // cmdline
-  sprintf(b2,"/proc/%d/cmdline",whichpid);
+  snprintf(b2,128,"/proc/%d/cmdline",whichpid);
+  b2[127]=0;
   f=fopen(b2,"r");
   if (f!=NULL) {
     memset(b3,0,512);
@@ -472,7 +501,7 @@ void LinuxProcFsProcessDetailsPoller::poll(int whichpid) {
 
   // start time
   strcpy(b2,t->token(" \t\n"));
-  rtime=(unsigned long)atol(b2);
+  rtime=gps_atoul(b2);
   ptime=btime+(time_t)(rtime/100UL);
   strcpy(b3,ctime(&ptime));
   if (b3[strlen(b3)-1]=='\n')
@@ -485,9 +514,10 @@ void LinuxProcFsProcessDetailsPoller::poll(int whichpid) {
 
   // RSS
   strcpy(b2,t->token(" \t\n"));
-  rssi=atol(b2);
+  rssi=gps_atoul(b2);
   rssi<<=PAGE_SHIFT;
-  sprintf(b2,"%lu",rssi);
+  snprintf(b2,128,"%lu",rssi);
+  b2[127]=0;
   item->setField(RSS,b2);
 
   // RLIM
@@ -716,9 +746,9 @@ void LinuxProcFsSystemInfoProvider::update_cpu() {
       busy=0;
       for(i=0;i<3;i++) {
 	p=t.token(T_SNT);
-	busy+=(p==NULL)?0:atol(p);
+	busy+=(p==NULL)?0:gps_atoul(p);
       }
-      idle=atol(t.token(T_SNT));      
+      idle=gps_atoul(t.token(T_SNT));      
       if (last_busy==-1) {
 	last_busy=busy;
 	last_idle=idle;
@@ -751,9 +781,9 @@ void LinuxProcFsSystemInfoProvider::update_cpu() {
       busy=0;
       for(i=0;i<3;i++) {
 	p=t.token(T_SNT);
-	busy+=(p==NULL)?0:atol(p);
+	busy+=(p==NULL)?0:gps_atoul(p);
       }
-      idle=atol(t.token(T_SNT));      
+      idle=gps_atoul(t.token(T_SNT));      
       if (lbusy[curcpu]==-1) {
 	lbusy[curcpu]=busy;
 	lidle[curcpu]=idle;
@@ -818,12 +848,12 @@ void LinuxProcFsSystemInfoProvider::update_memory() {
   t->set(buffer);
   t->token(T_SNT);
 
-  memory_total=atol(t->token(T_SNT));
-  memory_used=atol(t->token(T_SNT));
+  memory_total=gps_atoul(t->token(T_SNT));
+  memory_used=gps_atoul(t->token(T_SNT));
   t->token(T_SNT);
   t->token(T_SNT);
-  memory_used-=atol(t->token(T_SNT)); // buffers
-  memory_used-=atol(t->token(T_SNT)); // cached
+  memory_used-=gps_atoul(t->token(T_SNT)); // buffers
+  memory_used-=gps_atoul(t->token(T_SNT)); // cached
 
   memory_free=memory_total-memory_used; // Ohhhhhhhh...
 
@@ -845,9 +875,17 @@ void LinuxProcFsSystemInfoProvider::update_memory() {
 
   t->set(buffer);
   t->token(T_SNT);
-  swap_total=atol(t->token(T_SNT));
-  swap_used=atol(t->token(T_SNT));
+  swap_total=gps_atoul(t->token(T_SNT));
+  swap_used=gps_atoul(t->token(T_SNT));
   swap_free=swap_total-swap_used;
+
+  // test with ulf's 4-GB-swap-box values
+  /*
+  swap_total=4194840576;
+  swap_used=34021376;
+  swap_free=swap_total-swap_used;
+  */
+
   delete t;
 }
 
