@@ -1,9 +1,9 @@
-/* $Id: netpolling.cc,v 1.10 2000/09/29 10:15:37 bergo Exp $ */
+/* $Id: netpolling.cc,v 1.11 2002/03/10 18:37:10 bergo Exp $ */
 
 /*
 
     GPS - Graphical Process Statistics
-    Copyright (C) 1999-2000 Felipe Paulo Guazzi Bergo
+    Copyright (C) 1999-2002 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -139,7 +139,8 @@ int NetworkActor::hopen(char *hostname,int port) {
   is_special=0;
 
   if (gotgui) {
-    sprintf(buffer,"Looking up host %s...",hostname);
+    snprintf(buffer,512,"Looking up host %s...",hostname);
+    buffer[511]=0;
     gtk_label_set_text(GTK_LABEL(status_label[1]),buffer);
     while(gtk_events_pending()) gtk_main_iteration();
   }
@@ -161,10 +162,12 @@ int NetworkActor::hopen(char *hostname,int port) {
 	lwid=what_im_doing;
 	switch(what_im_doing) {
 	case 0:
-	  sprintf(buffer,"Looking up host %s...",hostname);
+	  snprintf(buffer,512,"Looking up host %s...",hostname);
+	  buffer[511]=0;
 	  break;
 	case 1:
-	  sprintf(buffer,"Opening connection to %s...",hostname);
+	  snprintf(buffer,512,"Opening connection to %s...",hostname);
+	  buffer[511]=0;
 	  break;
 	}
 	gtk_label_set_text(GTK_LABEL(status_label[1]),buffer);
@@ -189,9 +192,9 @@ int NetworkActor::hopen(char *hostname,int port) {
 
 	already_in=0;
 	if (net_halt_request)
-	  sprintf(buffer,"Failed! User abort.",hostname);
+	  strcpy(buffer,"Failed! User abort.");
 	else
-	  sprintf(buffer,"Failed! Connection timed out.",hostname);
+	  strcpy(buffer,"Failed! Connection timed out.");
 	gtk_label_set_text(GTK_LABEL(status_label[1]),buffer);
 	while(gtk_events_pending()) gtk_main_iteration();
 	usleep(400000);
@@ -280,7 +283,8 @@ void NetworkActor::request_detailed_refresh() {
     return;
   }
   if (ok<0) return;
-  sprintf(y,"%c",12);
+  snprintf(y,4,"%c",12);
+  y[3]=0;
   write(thesocket,y,strlen(y)); // ask for detailed refresh
 }
 
@@ -396,7 +400,8 @@ void NetworkListPoller::poll() {
 
     /* CPU (WCPU in top) */
     strcpy(b2,t->token(" %\n\t"));
-    sprintf(b3,"%.1f",atof(b2));
+    snprintf(b3,MAX_NET_LINE,"%.1f",atof(b2));
+    b3[MAX_NET_LINE-1]=0;
     pli->setField(CPU,b3);    
 
     strcpy(b2,t->token(" \n\t"));
@@ -616,7 +621,8 @@ void NetworkDetailsPoller::queryBuffer(int whichpid) {
     
     /* CPU (WCPU in top) */
     strcpy(b2,t->token(" %\t\n"));
-    sprintf(b3,"%.1f",atof(b2));
+    snprintf(b3,MAX_NET_LINE,"%.1f",atof(b2));
+    b3[MAX_NET_LINE-1]=0;
     item->setField(CPU,b3);    
     
     strcpy(b2,t->token(" \n\t"));
@@ -817,7 +823,8 @@ void NetworkDetailsPoller::poll(int whichpid) {
     
     /* CPU (WCPU in top) */
     strcpy(b2,t->token(" %\t\n"));
-    sprintf(b3,"%.1f",atof(b2));
+    snprintf(b3,MAX_NET_LINE,"%.1f",atof(b2));
+    b3[MAX_NET_LINE-1]=0;
     item->setField(CPU,b3);    
     
     strcpy(b2,t->token(" \n\t"));
