@@ -1,9 +1,9 @@
-/* $Id: netwatch.cc,v 1.6 2000/04/15 14:12:33 bergo Exp $ */
+/* $Id: netwatch.cc,v 1.8 2002/03/10 18:37:10 bergo Exp $ */
 
 /*
 
     GPS - Graphical Process Statistics
-    Copyright (C) 1999-2000 Felipe Paulo Guazzi Bergo
+    Copyright (C) 1999-2002 Felipe Paulo Guazzi Bergo
     bergo@seul.org
 
     This program is free software; you can redistribute it and/or modify
@@ -52,8 +52,8 @@ static char yesno[2][5]={"Yes","No"};
 static char conn_state[3][10]={"-","OK","FAILED"};
 
 void netwatch_config(GtkWidget * widget,gpointer data) {
-  GtkWidget *v1,*h1,*h2,*h3,*sw,*lb1,*cmd[10],*sep,*h4,
-    *ch1,*ch2;
+  GtkWidget *v1,*h1,*h2,*h3,*sw,*lb1,*cmd[10],*sep,*h4;
+
   int i;
   GList *pt;
   WatchedHost *wh;
@@ -130,15 +130,12 @@ void netwatch_config(GtkWidget * widget,gpointer data) {
   sep=gtk_hseparator_new();
   gtk_box_pack_start(GTK_BOX(v1),sep,FALSE,TRUE,2);
   
-  h4=gtk_hbox_new(TRUE,2);
-  gtk_box_pack_start(GTK_BOX(v1),h4,FALSE,TRUE,2);
-  
-  ch1=gtk_hbox_new(TRUE,2);
-  gtk_box_pack_start(GTK_BOX(h4),ch1,FALSE,TRUE,2);
-  
-  ch2=gtk_hbox_new(TRUE,8);
-  gtk_box_pack_start(GTK_BOX(h4),ch2,FALSE,TRUE,2);
-  
+  h4=gtk_hbutton_box_new();
+  gtk_button_box_set_layout(GTK_BUTTON_BOX(h4), GTK_BUTTONBOX_END);
+  gtk_button_box_set_spacing(GTK_BUTTON_BOX(h4), 5);
+
+  gtk_box_pack_start(GTK_BOX(v1),h4,FALSE,FALSE,6);
+    
   cmd[0]=gtk_button_new_with_label("Add host");
   gtk_box_pack_start(GTK_BOX(h2),cmd[0],FALSE,TRUE,0);
   
@@ -159,13 +156,16 @@ void netwatch_config(GtkWidget * widget,gpointer data) {
   
   
   cmd[6]=gtk_button_new_with_label("Ok");
-  gtk_box_pack_start(GTK_BOX(ch2),cmd[6],FALSE,TRUE,0);
+  GTK_WIDGET_SET_FLAGS(cmd[6],GTK_CAN_DEFAULT);
+  gtk_box_pack_start(GTK_BOX(h4),cmd[6],TRUE,TRUE,0);
   
   cmd[7]=gtk_button_new_with_label("Cancel");
-  gtk_box_pack_start(GTK_BOX(ch2),cmd[7],FALSE,TRUE,0);
+  GTK_WIDGET_SET_FLAGS(cmd[7],GTK_CAN_DEFAULT);
+  gtk_box_pack_start(GTK_BOX(h4),cmd[7],TRUE,TRUE,0);
   
   cmd[8]=gtk_button_new_with_label("Help");
-  gtk_box_pack_start(GTK_BOX(ch2),cmd[8],FALSE,TRUE,0);
+  GTK_WIDGET_SET_FLAGS(cmd[8],GTK_CAN_DEFAULT);
+  gtk_box_pack_start(GTK_BOX(h4),cmd[8],TRUE,TRUE,0);
   
   gtk_widget_set_sensitive(toggle_button,FALSE);
   gtk_widget_set_sensitive(remove_button,FALSE);
@@ -183,8 +183,7 @@ void netwatch_config(GtkWidget * widget,gpointer data) {
 		     GTK_SIGNAL_FUNC(enable_all_netwatch),NULL);
   gtk_signal_connect(GTK_OBJECT(cmd[5]),"clicked",
 		     GTK_SIGNAL_FUNC(disable_all_netwatch),NULL);
-  
-  
+    
   gtk_signal_connect(GTK_OBJECT(cmd[6]),"clicked",
 		     GTK_SIGNAL_FUNC(ok_netwatch),NULL);  
   gtk_signal_connect(GTK_OBJECT(cmd[7]),"clicked",
@@ -197,6 +196,8 @@ void netwatch_config(GtkWidget * widget,gpointer data) {
   gtk_signal_connect (GTK_OBJECT(pl),"unselect_row",
 		      GTK_SIGNAL_FUNC(netwatch_deselect),NULL);
   
+  gtk_widget_grab_default(cmd[6]);
+
   gtk_widget_show(v1);
   gtk_widget_show(lb1);
   gtk_widget_show(h1);
@@ -210,10 +211,7 @@ void netwatch_config(GtkWidget * widget,gpointer data) {
   gtk_widget_show(h2);
   gtk_widget_show(h3);
   
-  gtk_widget_show(h4);
-  gtk_widget_show(ch1);
-  gtk_widget_show(ch2);
-  
+  gtk_widget_show(h4);  
   gtk_widget_show(sep);
   
   gtk_widget_show(nwdlg);
@@ -274,15 +272,21 @@ void help_netwatch(GtkWidget *wid, gpointer data) {
   sep=gtk_hseparator_new();
   gtk_box_pack_start(GTK_BOX(v1),sep,FALSE,TRUE,2);
   
-  bh=gtk_hbox_new(FALSE,2);
-  gtk_box_pack_start(GTK_BOX(v1),bh,FALSE,TRUE,2);
+  bh=gtk_hbutton_box_new();
+  gtk_button_box_set_layout(GTK_BUTTON_BOX(bh), GTK_BUTTONBOX_END);
+  gtk_button_box_set_spacing(GTK_BUTTON_BOX(bh), 5);
 
-  dis=gtk_button_new_with_label(" Dismiss ");
-  gtk_box_pack_end(GTK_BOX(bh),dis,FALSE,FALSE,2);
+  gtk_box_pack_start(GTK_BOX(v1),bh,FALSE,FALSE,6);
+
+  dis=gtk_button_new_with_label("Dismiss");
+  GTK_WIDGET_SET_FLAGS(dis,GTK_CAN_DEFAULT);
+  gtk_box_pack_start(GTK_BOX(bh),dis,TRUE,TRUE,0);
 
   gtk_signal_connect(GTK_OBJECT(dis),"clicked",
 		     GTK_SIGNAL_FUNC(netwatch_help_dismiss),NULL);
 
+  gtk_widget_grab_default(dis);
+
   gtk_widget_show(dis);
   gtk_widget_show(landp);
   gtk_widget_show(bh);
@@ -444,7 +448,8 @@ void update_netwatch_hostlist() {
     p[0]=wh->hostname;
     p[2]=(wh->enable)?yesno[0]:yesno[1];
     
-    sprintf(b,"%d",wh->tcpport);
+    snprintf(b,32,"%d",wh->tcpport);
+    b[31]=0;
     p[1]=b;
 
     if (wh->attached_poller==NULL) {
